@@ -2,9 +2,10 @@
 #this repository contains the full copyright notices and license terms.
 "Statement"
 from trytond.model import ModelWorkflow, ModelView, ModelSQL, fields
+from trytond.pyson import Not, Equal, Eval, Or, Bool, Get, If
 from decimal import Decimal
 
-_STATES = {'readonly': 'state != "draft"'}
+_STATES = {'readonly': Not(Equal(Eval('state'), 'draft'))}
 
 
 class Statement(ModelWorkflow, ModelSQL, ModelView):
@@ -14,18 +15,22 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
 
     journal = fields.Many2One('account.statement.journal', 'Journal', required=True,
             states={
-                'readonly': "(state != 'draft') or (bool(lines))",
+                'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                    Bool(Eval('lines'))),
             }, on_change=['journal'], select=1)
-    currency_digits = fields.Function('get_currency_digits', type='integer',
-            string='Currency Digits', on_change_with=['journal'])
+    currency_digits = fields.Function(fields.Integer('Currency Digits',
+        on_change_with=['journal']), 'get_currency_digits')
     date = fields.Date('Date', required=True, states=_STATES, select=1)
-    start_balance = fields.Numeric('Start Balance', digits="(16, currency_digits)",
+    start_balance = fields.Numeric('Start Balance',
+            digits=(16, Eval('currency_digits', 2)),
             states=_STATES, depends=['currency_digits'])
-    end_balance = fields.Numeric('End Balance', digits="(16, currency_digits)",
+    end_balance = fields.Numeric('End Balance',
+            digits=(16, Eval('currency_digits', 2)),
             states=_STATES, depends=['currency_digits'])
     lines = fields.One2Many('account.statement.line', 'statement',
             'Transactions', states={
-                'readonly': "(state != 'draft') or (not bool(journal))",
+                'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                    Not(Bool(Eval('journal')))),
             }, on_change=['lines', 'journal'])
     state = fields.Selection([
         ('draft', 'Draft'),
@@ -33,8 +38,8 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
         ('cancel', 'Canceled'),
         ('posted', 'Posted'),
         ], 'State', readonly=True, select=1)
-    move_lines = fields.Function('get_move_lines', type='one2many',
-            relation='account.move.line', string='Move Lines')
+    move_lines = fields.Function(fields.One2Many('account.move.line',
+        None, 'Move Lines'), 'get_move_lines')
 
     def __init__(self):
         super(Statement, self).__init__()
@@ -63,7 +68,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
             return company.currency.digits
         return 2
 
-    def on_change_journal(self, cursor, user, ids, value, context=None):
+    def on_change_journal(self, cursor, user, value, context=None):
         res = {}
         if not value.get('journal'):
             return res
@@ -80,8 +85,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
         res['start_balance'] = statement.end_balance
         return res
 
-    def on_change_with_currency_digits(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_currency_digits(self, cursor, user, vals, context=None):
         journal_obj = self.pool.get('account.statement.journal')
         if vals.get('journal'):
             journal = journal_obj.browse(cursor, user, vals['journal'],
@@ -89,13 +93,13 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
             return journal.currency.digits
         return 2
 
-    def get_currency_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_currency_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for statement in self.browse(cursor, user, ids, context=context):
             res[statement.id] = statement.journal.currency.digits
         return res
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         lang_obj = self.pool.get('ir.lang')
 
         if context is None:
@@ -123,22 +127,16 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                         grouping=True)
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, ['OR',
-                ('start_balance', args[i][1], args[i][2]),
-                ('end_balance', args[i][1], args[i][2]),
-                ], context=context)
-            if ids:
-                args2.append(('id', 'in', ids))
-            else:
-                args2.append(('journal', args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, ['OR',
+            ('start_balance',) + clause[1:],
+            ('end_balance',) + clause[1:],
+            ], context=context)
+        if ids:
+            return [('id', 'in', ids)]
+        return [('journal',) + clause[1:]]
 
-    def get_move_lines(self, cursor, user, ids, name, args, context=None):
+    def get_move_lines(self, cursor, user, ids, name, context=None):
         '''
         Return the move lines that have been generated by the statements.
         '''
@@ -152,7 +150,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                     res[statement.id].append(move_line.id)
         return res
 
-    def get_end_balance(self, cursor, user, ids, name, arg, context=None):
+    def get_end_balance(self, cursor, user, ids, name, context=None):
         statements = self.browse(cursor, user, ids, context=context)
         res = {}
         for statement in statements:
@@ -161,7 +159,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                 res[statement.id] += line.amount
         return res
 
-    def on_change_lines(self, cursor, user, ids, values, context=None):
+    def on_change_lines(self, cursor, user, values, context=None):
         invoice_obj = self.pool.get('account.invoice')
         journal_obj = self.pool.get('account.statement.journal')
         currency_obj = self.pool.get('currency.currency')
@@ -281,7 +279,8 @@ class Line(ModelSQL, ModelView):
             required=True, ondelete='CASCADE')
     date = fields.Date('Date', required=True)
     amount = fields.Numeric('Amount', required=True,
-            digits="(16, _parent_statement.currency_digits)",
+            digits=(16, Get(Eval('_parent_statement', {}),
+                'currency_digits', 2)),
             on_change=['amount', 'party', 'account', 'invoice',
                 '_parent_statement.journal'])
     party = fields.Many2One('party.party', 'Party',
@@ -291,11 +290,14 @@ class Line(ModelSQL, ModelView):
     description = fields.Char('Description')
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
     invoice = fields.Many2One('account.invoice', 'Invoice',
-            domain=["('party', '=', party)", "('account', '=', account)",
-                "('state', '=', _parent_statement.state == 'draft' and " \
-                        "'open' or False)"],
+            domain=[
+                ('party', '=', Eval('party')),
+                ('account', '=', Eval('account')),
+                ('state', '=', If(Equal(Get(Eval('_parent_statement', {}),
+                    'state'), 'draft'), 'open', '')),
+            ],
             states={
-                'readonly': "not bool(amount)",
+                'readonly': Not(Bool(Eval('amount'))),
             })
 
     def __init__(self):
@@ -309,7 +311,7 @@ class Line(ModelSQL, ModelView):
                     'the amount to pay of invoice!',
             })
 
-    def on_change_party(self, cursor, user, ids, value, context=None):
+    def on_change_party(self, cursor, user, value, context=None):
         party_obj = self.pool.get('party.party')
         account_obj = self.pool.get('account.account')
         invoice_obj = self.pool.get('account.invoice')
@@ -336,7 +338,7 @@ class Line(ModelSQL, ModelView):
                 res['invoice'] = False
         return res
 
-    def on_change_amount(self, cursor, user, ids, value, context=None):
+    def on_change_amount(self, cursor, user, value, context=None):
         party_obj = self.pool.get('party.party')
         account_obj = self.pool.get('account.account')
         invoice_obj = self.pool.get('account.invoice')
@@ -372,7 +374,7 @@ class Line(ModelSQL, ModelView):
                 res['invoice'] = False
         return res
 
-    def on_change_account(self, cursor, user, ids, value, context=None):
+    def on_change_account(self, cursor, user, value, context=None):
         invoice_obj = self.pool.get('account.invoice')
         res = {}
 
